import 'dart:async';
import 'dart:io';
import 'package:dio/dio.dart';
import 'package:test/test.dart';

void main() {
  const BASE_URL = "http://www.dtworkroom.com/doris/1/2.0.0/";
  group('lan', () {
    test("lan", (){

      //print(data);
    });
  });
  group('request', () {
    Dio dio;
    setUp(() {
      dio = new Dio();
      dio.options.baseUrl = BASE_URL;
      dio.options.headers = {
        'User-Agent': 'dartisan',
        'XX': '8'
      };
      dio.options.contentType=ContentType.parse("application/x-www-form-urlencoded");

    });

    test('Test GET request 404', () async {
      dio.httpClient.findProxy = (uri) {
        print(uri);
        return "PROXY localhost:8888";
      };
      //p path is not exist.
      Response response = await dio.get("/path",data:{"a":5,"b":9,"c":{"a":5,"b":"xx"}} );

      expect(response.data["errCode"], 666);
    });
    test('Test GET request ', () async {
      Response response = await dio.get("/test");
      expect(response.data["errCode"], 0);
    });

    test('Test POST request', () async {
      Response response = await dio.post("/test");
      expect(response.data["errCode"], 0);
    });

    test("concurrency",() async {
      var t= await Future.wait([dio.post("/test"),dio.post("/token")]);
      print(t);
    });

  });

  group('Request Interceptor', () {
    Dio dio;
    setUp(() {
      dio = new Dio();
      dio.options.baseUrl = BASE_URL;
      dio.interceptor.request.onSend = (Options options) async {
        switch (options.path) {
          case "/fakepath1":
            return dio.resolve("fake data");
          case "/fakepath2":
            return dio.get("/test");
          case "/fakepath3":
            return dio.reject("test error"); //you can also return a HttpError directly.
          case "fakepath4":
            return new DioError(message:"test error"); // Here is equivalent to call dio.reject("test error")
          case "/test?tag=1":
            {
              Response response = await dio.get("/token");
              options.headers["token"] = response.data["data"]["token"];
              return options;
            }
          default:
            return options; //continue
        }
      };
    });

    test('Test', () async {
      Response
      response = await dio.get("/fakepath1");
      expect(response.data, "fake data");
      response = await dio.get("/fakepath2");
      expect(response.data["errCode"], 0);

      try {
        response = await dio.get("/fakepath3");
      } on DioError catch (e) {
        expect(e.message, "test error");
        expect(e.response, null);
      }
      try {
        response = await dio.get("/fakepath4");
      } on DioError catch (e) {
        expect(e.message, "test error");
        expect(e.response, null);
      }
      response = await dio.get("/test");
      expect(response.data["errCode"], 0);

      response = await dio.get("/test?tag=1");
      expect(response.data["errCode"], 0);

      try {
        await dio.get("https://wendux.github.io/xsddddd");
      }  on DioError catch(e) {
        expect(e.response.statusCode, 404);
      }

    });
  });


  group('Response Interceptor', () {
    Dio dio;
    const String URL_NOT_FIND="https://wendux.github.io/xxxxx/";
    const String URL_NOT_FIND_1=URL_NOT_FIND+"1";
    const String URL_NOT_FIND_2=URL_NOT_FIND+"2";
    const String URL_NOT_FIND_3=URL_NOT_FIND+"3";
    const String URL_NOT_FIND_4=URL_NOT_FIND+"4";
    setUp(() {
      dio = new Dio();
      dio.options.baseUrl = BASE_URL;
      dio.interceptor.response.onSuccess = (Response response) {
        return response.data["data"]; //
      };
      dio.interceptor.response.onError = (DioError e) async {
        if (e.response != null) {
          switch (e.response.request.path){
            case URL_NOT_FIND: return e;
            case URL_NOT_FIND_1: return dio.resolve("fake data");// you can also return a HttpError directly.
            case URL_NOT_FIND_2: return new Response(data: "fake data");
            case URL_NOT_FIND_3: return 'custom error info [${e.response.statusCode}]';
          }
        }
        return e;
      };
    });

    test('Test', () async {
      Response
      response = await dio.get("/test");
      expect(response.data["path"], "/test");
      try {
        await dio.get(URL_NOT_FIND);
      } catch (e) {
        expect(e.response.statusCode, 404);
      }
      response = await dio.get(URL_NOT_FIND+"1");
      expect(response.data, "fake data");
      response = await dio.get(URL_NOT_FIND+"2");
      expect(response.data, "fake data");
      try {
        await dio.get(URL_NOT_FIND+"3");
      } catch (e) {
        expect(e.message, 'custom error info [404]');
      }
    });
  });
}
